#include "hope.h"

void init(void)
{
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_FLAT);
}

void display(void)
{
  glClear(GL_COLOR_BUFFER_BIT);
  glColor3f(1.0, 1.0, 1.0);
  glLoadIdentity();
  glTranslatef(p1.x, p1.y, p1.z);
  glRotatef(p1.roll, 0.0, 0.0, 1.0);
  glRotatef(p1.pitch, 0.0, 1.0, 0.0);
  glRotatef(p1.yaw, 1.0, 0.0, 0.0);
  gluLookAt (eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ);
  glScalef(1.0, 1.0, 1.0);
  
  int x,y,z;
  for (x = 0; x < d; x++)
  {
    for (y = 0; y < d; y++)
    {
      for (z = 0; z < d; z++)
      {
	if (v[x][y][z])
	{
	  glPushMatrix();
	  glTranslatef(x, y, z);
	  glutSolidCube(1.0);
	  glPopMatrix();
	}
      }
    }
  }
  glFlush();
}

void reshape (int w, int h)
{
  glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
  glMatrixMode (GL_MODELVIEW);
}

void keyboard(unsigned char key, int x, int y)
{
  switch (key)
  {
    case 'w': p1.z += 0.1; break;
    case 'a': p1.x += 0.1; break;
    case 's': p1.z -= 0.1; break;
    case 'd': p1.x -= 0.1; break;
    case ' ': p1.y -= 0.1; break;
    case 'c': p1.y += 0.1; break;
    case 'o': p1.pitch += 1; break;
    case 'l': p1.pitch -= 1; break;
    case 'r': p1.yaw += 1; break;
    case 'u': p1.yaw -= 1; break;
    case 'k': p1.roll += 1; break;
    case ';': p1.roll -= 1; break;
    case 27:
      exit(EXIT_SUCCESS);
      break;
  }
  glutPostRedisplay();
}

int main (int argc, char * argv[])
{
  int x,y,z;
  
  bool i[d][d];
  fractInit(d, i);
  
  for (y=0;y<d;y++)
  {
    for (x=0;x<d;x++)
    {
      for (z=0;z<d;z++)
      {
	v[x][y][z] = (i[x][y] && i[z][y]);
      }
    }
  }
  
  glutInit(&argc, argv);
  glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
  glutInitWindowSize (500, 500); 
  glutInitWindowPosition (100, 100);
  glutCreateWindow (argv[0]);
  init ();
  glutDisplayFunc(display); 
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard);
  glutMainLoop();
  
  return(EXIT_SUCCESS);
}

void fractInit(int size, bool i[][size])
{
  int x, y;
  for (x = 0; x < size; x++)
  {
    for (y = 0; y < size; y++)
    {
      i[x][y] = ((x % 3) == 0) && ((y % 4) == 0);
    }
  }
}

/*
  Convention for writing cubes is:
    Bottom of cube: 0 1 2 3 where 0 is (0,0,0)
    Top of cube: 7 6 5 4 where 5 is (1,1,1)
  The order of operations is significant when Blender
  tries to read the .obj output file. Stay safe.
 */
void writeCube(FILE * fout, int x, int y, int z, int r)
{
  fprintf(fout, "v %d %d %d\n", x + 0, y + 0, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + 0, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + r, z + 0);
  fprintf(fout, "v %d %d %d\n", x + 0, y + r, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + r, z + r);
  fprintf(fout, "v %d %d %d\n", x + r, y + 0, z + r);
  fprintf(fout, "v %d %d %d\n", x + 0, y + 0, z + r);
  fprintf(fout, "v %d %d %d\n", x + 0, y + r, z + r);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 3, totalVerts + 2, totalVerts + 1, totalVerts + 0);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 7, totalVerts + 6, totalVerts + 5, totalVerts + 4);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 0, totalVerts + 1, totalVerts + 6, totalVerts + 7);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 3, totalVerts + 0, totalVerts + 7, totalVerts + 4);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 2, totalVerts + 3, totalVerts + 4, totalVerts + 5);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 1, totalVerts + 2, totalVerts + 5, totalVerts + 6);
  totalVerts++;
}

void writeVertex(FILE * fout, int x, int y, int z)
{
  fprintf(fout, "v %d %d %d\n", x, y, z);
}
